#ifndef B_CAS_CARD_H
#define B_CAS_CARD_H

#include "portable.h"

typedef struct {
	uint8_t  system_key[32];
	uint8_t  init_cbc[8];
	int64_t  bcas_card_id;
	int32_t  card_status;
	int32_t  ca_system_id;
} B_CAS_INIT_STATUS;

typedef struct {
	uint8_t  maker_id;   /*  8 bit       */
	uint8_t  version;    /*  8 bit       */
	uint8_t  card_type;  /*  3 bit       */
	uint64_t card_id;    /* 45 bit       */
	uint16_t check_code; /* 16 bit       */
} B_CAS_CARD_ID_DATA;

typedef struct {
	B_CAS_CARD_ID_DATA *data;
	int32_t  count;
} B_CAS_CARD_ID_INFO;

typedef struct {
	
	int32_t  s_yy; /* start date : year  */
	int32_t  s_mm; /* start date : month */
	int32_t  s_dd; /* start date : day   */
	
	int32_t  l_yy; /* limit date : year  */
	int32_t  l_mm; /* limit date : month */
	int32_t  l_dd; /* limit date : day   */

	int32_t  hold_time; /* in hour unit  */

	int32_t  broadcaster_group_id;
	
	int32_t  network_id;
	int32_t  transport_id;
	
} B_CAS_PWR_ON_CTRL;

typedef struct {
	B_CAS_PWR_ON_CTRL *data;
	int32_t  count;
} B_CAS_PWR_ON_CTRL_INFO;

typedef struct {
	uint8_t  scramble_key[16];
	uint32_t return_code;
} B_CAS_ECM_RESULT;

typedef struct {
	uint8_t *cvi; // contract verification info
	uint32_t cvi_length;
	uint16_t mjd_date;
} B_CAS_CVI_DATA;

typedef struct {

	uint16_t  mjd_date;             // In - current date
	uint8_t   broadcaster_group_id; // In 
	uint8_t   suspend_time;         // In
	
	uint16_t  expired_limit;        // Out 
	uint16_t  message_format_num;   // Out
	uint8_t   diff_format_num;      // Out
	uint16_t  diff_info_length;     // Out
	uint8_t  *diff_info;            // Out      
	
} B_CAS_MESSAGE_DATA;

typedef struct {

	void *private_data;

	void (* release)(void *bcas);

	int (* init)(void *bcas);

	int (* get_init_status)(void *bcas, B_CAS_INIT_STATUS *stat);
	int (* get_card_id)(void *bcas, B_CAS_CARD_ID_INFO *dst);
	int (* get_pwr_on_ctrl)(void *bcas, B_CAS_PWR_ON_CTRL_INFO *dst);
	int (* get_message)(void *bcas, B_CAS_MESSAGE_DATA *dat);
	
	int (* check_contract_status)(void *bcas, B_CAS_CVI_DATA *dat);

	int (* proc_ecm)(void *bcas, B_CAS_ECM_RESULT *dst, uint8_t *src, int len);
	int (* proc_emm)(void *bcas, uint8_t *src, int len);
	
} B_CAS_CARD;

#ifdef __cplusplus
extern "C" {
#endif

extern B_CAS_CARD *create_b_cas_card();

#ifdef __cplusplus
}
#endif

#endif /* B_CAS_CARD_H */
