#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "b_cas_card.h"
#include "b_cas_card_error_code.h"

#include "broadcast_service.h"

#include "arib_util.h"

static void dump_init_status(B_CAS_INIT_STATUS *stat);
static void dump_card_id(B_CAS_CARD_ID_INFO *id);
static void dump_poc(B_CAS_PWR_ON_CTRL_INFO *poc);

static void dump_contract_period(B_CAS_CARD *bcas, BROADCAST_SERVICE_INFO *bsi);

int main(int argc, char **argv)
{
	B_CAS_INIT_STATUS       stat;
	B_CAS_PWR_ON_CTRL_INFO  poc;
	B_CAS_CARD_ID_INFO      id;
	
	B_CAS_CARD             *bcas = NULL;

	int i,n;

	bcas = create_b_cas_card();
	if (bcas == NULL) {
		fprintf(stderr, "error - failed on create_b_cas_card()\n");
		goto LAST;
	}

	n = bcas->init(bcas);
	if (n < 0) {
		fprintf(stderr, "error - failed on B_CAS_CARD::init() : code=%d\n", n);
		goto LAST;
	}

	n = bcas->get_init_status(bcas, &stat);
	if (n < 0) {
		fprintf(stderr, "error - failed on B_CAS_CARD::get_init_status() : code=%d\n", n);
		goto LAST;
	}

	dump_init_status(&stat);

	n = bcas->get_card_id(bcas, &id);
	if (n < 0) {
		fprintf(stderr, "error - failed on B_CAS_CARD::get_card_id() : code=%d\n", n);
		goto LAST;
	}

	dump_card_id(&id);

	n = bcas->get_pwr_on_ctrl(bcas, &poc);
	if (n < 0) {
		fprintf(stderr, "error - failed on B_CAS_CARD::get_pwr_on_ctrl() : code=%d\n", n);
		goto LAST;
	}

	dump_poc(&poc);

	fprintf(stdout, "----\n");
	for (i=0;i<sizeof(bsi_table)/sizeof(bsi_table[0]);i++) {
		dump_contract_period(bcas, bsi_table+i);
	}

LAST:
	if (bcas != NULL) {
		bcas->release(bcas);
		bcas = NULL;
	}

	return EXIT_SUCCESS;
}

static void dump_init_status(B_CAS_INIT_STATUS *stat)
{
	int i;
	
	fprintf(stdout, "system_key: \n");
	for (i=0;i<32;i++) {
		if ((i&7) == 0) {
			fprintf(stdout, "    ");
		}
		fprintf(stdout, "%02x", stat->system_key[i]);
		if ((i&7) == 7) {
			fprintf(stdout, "\n");
		}
	}

	fprintf(stdout, "init_cbc:\n");
	for (i=0;i<8;i++) {
		if ((i&7) == 0) {
			fprintf(stdout, "    ");
		}
		fprintf(stdout, "%02x", stat->init_cbc[i]);
		if ((i&7) == 7) {
			fprintf(stdout, "\n");
		}
	}

	fprintf(stdout, "bcas_card_id: %I64d\n", stat->bcas_card_id);
	fprintf(stdout, "card_status: %04x\n", stat->card_status);
	fprintf(stdout, "ca_system_id: %04x\n", stat->ca_system_id);
}

static void dump_card_id(B_CAS_CARD_ID_INFO *id)
{
	int i;
	
	for (i=0;i<id->count;i++) {
		fprintf(stdout, "card_id[%d] = {\n");
		fprintf(stdout, "  type: %c%03d\n", id->data[i].maker_id, id->data[i].version);
		fprintf(stdout, "  card_type: %d\n", id->data[i].card_type);
		fprintf(stdout, "  card_id: %I64d\n", id->data[i].card_id);
		fprintf(stdout, "  check_code: %d\n", id->data[i].check_code);
		fprintf(stdout, "}\n");
	}
}

static void dump_poc(B_CAS_PWR_ON_CTRL_INFO *poc)
{
	int i;
	char tag[1024];

	fprintf(stdout, "----\n");
	
	if (poc->count == 0) {
		fprintf(stdout, "no power_on_control\n");
		return;
	}

	fprintf(stdout, "total %d power_on_control\n", poc->count);
	for(i=0;i<poc->count;i++){
		fprintf(stdout, "+ [%d] : tune ", i);
		extract_transport_stream_id(tag, poc->data[i].transport_id);
		fprintf(stdout, "%s ", tag);
		fprintf(stdout, "between %04d %02d/%02d ", poc->data[i].s_yy, poc->data[i].s_mm, poc->data[i].s_dd);
		fprintf(stdout, "to %04d %02d/%02d ", poc->data[i].l_yy, poc->data[i].l_mm, poc->data[i].l_dd);
		fprintf(stdout, "least %d hours\n", poc->data[i].hold_time);
	}
}

static void dump_contract_period(B_CAS_CARD *bcas, BROADCAST_SERVICE_INFO *bsi)
{
	int n;
	int yy,mm,dd;

	char tag[1024];
	
	uint16_t cur,min,max;
	B_CAS_CVI_DATA dat;
	
	min = 51604; // 2000,3/1
	max = 65535;
	
	dat.cvi = bsi->cvi;
	dat.cvi_length = bsi->cvi_length;
	dat.mjd_date = min;

	extract_transport_stream_id(tag, bsi->transport_stream_id);

	n = bcas->check_contract_status(bcas, &dat);
	if (n < 0) {
		fprintf(stderr, "error - failed on B_CAS_CARD::check_contract_status(%s) : code=%d\n", bsi->name, n);
		return;
	} else if (n != 0) {
		extract_mjd(&yy, &mm, &dd, dat.mjd_date);
		fprintf(stdout, "%24s : %s : %d : %4d/%02d/%02d : uncontracted\n", bsi->name, tag, bsi->service_id, yy, mm, dd);
		return;
	}

	dat.mjd_date = max;
	n = bcas->check_contract_status(bcas, &dat);
	if (n == 0) {
		extract_mjd(&yy, &mm, &dd, dat.mjd_date);
		fprintf(stdout, "%24s : %s : %d : %4d/%02d/%02d\n", bsi->name, tag, bsi->service_id, yy, mm, dd);
		return;
	}
	
	while ((max-min) > 1) {

		cur = ((max - min) / 2) + min;
		dat.mjd_date = cur;
		n = bcas->check_contract_status(bcas, &dat);

		if (n == 0) {
			min = cur;
		} else {
			max = cur;
		}
	}
	
	extract_mjd(&yy, &mm, &dd, min);
	fprintf(stdout, "%24s : %s : %d : %4d/%02d/%02d\n", bsi->name, tag, bsi->service_id, yy, mm, dd);
	fflush(stdout);
}
